''
'' single wire serial link - transmitter
''
''        Author: Marko Lukat
'' Last modified: 2010/11/24
''       Version: 0.2
''
OBJ
  system: "core.con.system"
  
PUB null
'' This is not a top level object.

PUB init(tx)

  return cognew(@transmit, tx) + 1

DAT             org     0

header_2048     long    system#ID_2             ' magic number for a cog binary
                word    header_size             ' header size
                word    %00000000_00000000      ' flags
                word    0, 0                    ' start register, register count

header_size     fit     16
                
CON
  shl_phsb_imm1 = $2CFFFA01                     ' shl phsb, #1
  
DAT             org     0

transmit        jmpret  $, #:setup              ' once

                rdlong  cnt, #0                 ' wait for receiver
                or      cnt, #3                 ' ... guarantee 4n ...
                add     cnt, cnt                ' to come up            (##)

                mov     data, seed              ' initialise LFSR

:loop           shl     data, #1 wc             ' |
        if_c    xor     data, seed              ' | advance LFSR

                waitcnt cnt, #176               ' frame sync (min 176) (##)

' prerequisites: ctra NCO clkfreq/4, low centres around phase D
'                ctrb NCO inactive, output preset to high (mutes ctra)

' transfer starts, 2 start bits, 32 data bits, 1 stop bit

                mov     phsb, #0                ' start bit 0
                neg     phsb, #1                ' start bit 1
                xor     phsb, data              ' bit 31
                long    shl_phsb_imm1[31]       ' bit 30..0
                neg     phsb, #1                ' stop bit

' transfer ends

                cmp     data, seed wz           '
        if_nz   jmp     #:loop                  ' continue until LFSR repeats

                cogid   cnt                     '
                cogstop cnt                     ' sayonara ...


:setup          rdbyte  ctra, par               ' read transmitter pin

                neg     phsb, #1                ' preset high
                movs    ctrb, ctra              ' copy pin assignment
                movi    ctrb, #%0_00100_000     ' NCO single-ended

                shl     mask, ctra              ' pin number -> pin mask
                mov     dira, mask              ' set output
                
                movi    ctra, #%0_00100_000     ' NCO single-ended
                movi    phsa, #%1100_0000_0     ' preset (low centres around phase D)
                movi    frqa, #%0100_0000_0     ' clkfreq/4

                wrlong  vscl, par               ' setup done
                jmp     $000                    ' ret
                
' initialised data and/or presets

mask            long    1                       ' pin mask/data
seed            long    $1D872B41               ' LFSR seed and xor mask

' uninitialised data and/or temporaries

data            res     1

                fit

DAT